@@ -112,12 +112,13 @@ extern NihList *conf_sources;
 
 
 void        conf_init          (void);
+void        conf_destroy       (void);
 
 ConfSource *conf_source_new    (const void *parent, const char *path,
 				ConfSourceType type)
-	__attribute__ ((warn_unused_result, malloc));
+	__attribute__ ((warn_unused_result));
 ConfFile *  conf_file_new      (ConfSource *source, const char *path)
-	__attribute__ ((warn_unused_result, malloc));
+	__attribute__ ((warn_unused_result));
 
 void        conf_reload        (void);
 int         conf_source_reload (ConfSource *source)
@@ -127,9 +128,6 @@ int         conf_file_destroy  (ConfFile *file);
 
 JobClass *  conf_select_job    (const char *name, const Session *session);
 
-char *toggle_conf_name         (const void *parent, const char *path)
-	__attribute__ ((warn_unused_result, malloc));
-
 #ifdef DEBUG
 
 /* used for debugging only */
