@@ -102,11 +102,11 @@ typedef enum console_type {
 #define JOB_DEFAULT_UMASK 022
 
 /**
- * JOB_DEFAULT_NICE:
+ * JOB_NICE_INVALID:
  *
- * The default nice level for processes.
+ * The nice level for processes when no nice level is set.
  **/
-#define JOB_DEFAULT_NICE 0
+#define JOB_NICE_INVALID -21
 
 /**
  * JOB_DEFAULT_OOM_SCORE_ADJ:
@@ -225,15 +225,27 @@ typedef struct job_class {
 
 NIH_BEGIN_EXTERN
 
-extern NihHash *job_classes;
-
+extern NihHash  *job_classes;
 
 void        job_class_init                 (void);
 
+void        job_class_environment_init     (void);
+
+void        job_class_environment_reset    (void);
+
+int         job_class_environment_set      (const char *var, int replace);
+int         job_class_environment_unset    (const char *name);
+
+char **     job_class_environment_get_all  (const void *parent)
+	__attribute__ ((warn_unused_result));
+
+const char *job_class_environment_get      (const char *name)
+	__attribute__ ((warn_unused_result));
+
 JobClass  * job_class_new                  (const void *parent,
 					    const char *name,
 					    Session *session)
-	__attribute__ ((warn_unused_result, malloc));
+	__attribute__ ((warn_unused_result));
 
 int         job_class_consider             (JobClass *class);
 int         job_class_reconsider           (JobClass *class);
@@ -247,7 +259,7 @@ void        job_class_unregister           (JobClass *class,
 
 char      **job_class_environment          (const void *parent,
 					    JobClass *class, size_t *len)
-	__attribute__ ((warn_unused_result, malloc));
+	__attribute__ ((warn_unused_result));
 
 
 int         job_class_get_instance         (JobClass *class,
@@ -328,13 +340,13 @@ ConsoleType job_class_console_type         (const char *console)
 	__attribute__ ((warn_unused_result));
 
 json_object *job_class_serialise (const JobClass *class)
-	__attribute__ ((warn_unused_result, malloc));
+	__attribute__ ((warn_unused_result));
 
 JobClass *job_class_deserialise (json_object *json)
-	__attribute__ ((malloc, warn_unused_result));
+	__attribute__ ((warn_unused_result));
 
 json_object * job_class_serialise_all (void)
-	__attribute__ ((warn_unused_result, malloc));
+	__attribute__ ((warn_unused_result));
 
 int job_class_deserialise_all (json_object *json)
 	__attribute__ ((warn_unused_result));
@@ -344,6 +356,12 @@ JobClass * job_class_get (const char *name, Session *session)
 
 void job_class_prepare_reexec (void);
 
+JobClass * job_class_find (const Session *session, const char *name)
+	__attribute__ ((warn_unused_result));
+
+time_t     job_class_max_kill_timeout (void)
+	__attribute__ ((warn_unused_result));
+
 NIH_END_EXTERN
 
 #endif /* INIT_JOB_CLASS_H */
