@@ -65,6 +65,8 @@
 #include "job_class.h"
 #include "job.h"
 #include "errors.h"
+#include "control.h"
+#include "xdg.h"
 
 
 /**
@@ -97,6 +99,13 @@ typedef struct job_process_wire_error {
  **/
 char *log_dir = NULL;
 
+/**
+ * disable_respawn:
+ *
+ * If TRUE, disallow respawning.
+ **/
+int disable_respawn = FALSE;
+
 /* Prototypes for static functions */
 static void job_process_error_abort     (int fd, JobProcessErrorType type,
 					 int arg)
@@ -113,6 +122,14 @@ static void job_process_remap_fd        (int *fd, int reserved_fd, int error_fd)
  **/
 int disable_job_logging = 0;
 
+/**
+ * no_inherit_env:
+ *
+ * If TRUE, do not copy the Session Inits environment to that provided to jobs.
+ **/
+int no_inherit_env = FALSE;
+
+
 /* Prototypes for static functions */
 static void job_process_kill_timer      (Job *job, NihTimer *timer);
 static void job_process_terminated      (Job *job, ProcessType process,
@@ -126,6 +143,10 @@ static void job_process_trace_signal    (Job *job, ProcessType process,
 static void job_process_trace_fork      (Job *job, ProcessType process);
 static void job_process_trace_exec      (Job *job, ProcessType process);
 
+extern char         *control_server_address;
+extern int           user_mode;
+extern int           session_end;
+extern time_t        quiesce_phase_time;
 
 /**
  * job_process_run:
@@ -239,6 +260,12 @@ job_process_run (Job         *job,
 			NIH_MUST (nih_str_array_addp (&argv, NULL,
 						      &argc, cmd));
 		}
+
+		/* At the end, always set proc->script to TRUE, even if the user didn't
+		 * explicitly set it (when using shell variables). That way tests
+		 * can reliably check for shell-specific behaviour.
+		 */
+		proc->script = TRUE;
 	} else {
 		/* Split the command on whitespace to produce a list of
 		 * arguments that we can exec directly.
@@ -253,11 +280,13 @@ job_process_run (Job         *job,
 	 * so that initctl can have clever behaviour when called within them.
 	 */
 	envc = 0;
-	if (job->env) {
-		env = NIH_MUST (nih_str_array_copy (NULL, &envc, job->env));
-	} else {
-		env = NIH_MUST (nih_str_array_new (NULL));
-	}
+	env = NIH_MUST (nih_str_array_new (NULL));
+
+	if (user_mode && ! no_inherit_env)
+		NIH_MUST(environ_append (&env, NULL, &envc, TRUE, environ));
+
+	if (job->env)
+		NIH_MUST(environ_append (&env, NULL, &envc, TRUE, job->env));
 
 	if (job->stop_env
 	    && ((process == PROCESS_PRE_STOP)
@@ -269,6 +298,9 @@ job_process_run (Job         *job,
 			       "UPSTART_JOB=%s", job->class->name));
 	NIH_MUST (environ_set (&env, NULL, &envc, TRUE,
 			       "UPSTART_INSTANCE=%s", job->name));
+	if (user_mode)
+		NIH_MUST (environ_set (&env, NULL, &envc, TRUE,
+			       "UPSTART_SESSION=%s", control_server_address));
 
 	/* If we're about to spawn the main job and we expect it to become
 	 * a daemon or fork before we can move out of spawned, we need to
@@ -407,12 +439,12 @@ job_process_spawn (Job          *job,
 	char            pts_name[PATH_MAX];
 	char            filename[PATH_MAX];
 	FILE           *fd;
-	int             user_job = FALSE;
-	nih_local char *user_dir = NULL;
 	nih_local char *log_path = NULL;
 	JobClass       *class;
 	uid_t           job_setuid = -1;
 	gid_t           job_setgid = -1;
+	struct passwd   *pwd = NULL;
+	struct group    *grp = NULL;
 
 
 	nih_assert (job != NULL);
@@ -424,20 +456,14 @@ job_process_spawn (Job          *job,
 
 	nih_assert (class != NULL);
 
-	if (class && class->session && class->session->user)
-		user_job = TRUE;
-
 	/* Create a pipe to communicate with the child process until it
 	 * execs so we know whether that was successful or an error occurred.
 	 */
 	if (pipe (fds) < 0)
 		nih_return_system_error (-1);
 
-	/* Logging of user job output is not currently possible */
-	if (class->console == CONSOLE_LOG) {
-		if (disable_job_logging || user_job)
+	if (class->console == CONSOLE_LOG && disable_job_logging)
 			class->console = CONSOLE_NONE;
-	}
 
 	if (class->console == CONSOLE_LOG) {
 		NihError *err;
@@ -640,82 +666,6 @@ job_process_spawn (Job          *job,
 	/* Set the process environment from the function parameters. */
 	environ = (char **)env;
 
-	/* Handle unprivileged user job by dropping privileges to
-	 * their level as soon as possible to avoid privilege
-	 * escalations when we set resource limits.
-	 */
-	if (user_job) {
-		uid_t uid = class->session->user;
-		struct passwd *pw = NULL;
-
-		/* D-Bus does not expose a public API call to allow
-		 * us to query a users primary group.
-		 * _dbus_user_info_fill_uid () seems to exist for this
-		 * purpose, but is a "secret" API. It is unclear why
-		 * D-Bus neglects the gid when it allows the uid
-		 * to be queried directly.
-		 *
-		 * Our only recourse is to disallow user sessions in a
-		 * chroot and assume that all other user sessions
-		 * originate from the local system. In this way, we can
-		 * bypass D-Bus and use getpwuid ().
-		 */
-
-		if (class->session->chroot) {
-			/* We cannot determine the group id of the user
-			 * session in the chroot via D-Bus, so disallow
-			 * all jobs in such an environment.
-			 */
-			nih_error_raise (EPERM, "user jobs not supported in chroots");
-			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_CHROOT, 0);
-		}
-
-		pw = getpwuid (uid);
-
-		if (!pw) {
-			nih_error_raise_system ();
-			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_GETPWUID, 0);
-		}
-
-		nih_assert (pw->pw_uid == uid);
-
-		if (! pw->pw_dir) {
-			nih_error_raise_printf (ENOENT,
-					"no home directory for user with uid %d", uid);
-			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_GETPWUID, 0);
-
-		}
-
-		/* Note we don't use NIH_MUST since this could result in a
-		 * DoS for a (low priority) user job in low-memory scenarios.
-		 */
-		user_dir = nih_strdup (NULL, pw->pw_dir);
-
-		if (! user_dir) {
-			nih_error_raise_no_memory ();
-			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_ALLOC, 0);
-		}
-
-		/* Ensure the file associated with fd 9
-		 * (/proc/self/fd/9) is owned by the user we're about to
-		 * become to avoid EPERM.
-		 */
-		if (script_fd != -1 && fchown (script_fd, pw->pw_uid, pw->pw_gid) < 0) {
-			nih_error_raise_system ();
-			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_CHOWN, 0);
-		}
-
-		if (pw->pw_gid && setgid (pw->pw_gid) < 0) {
-			nih_error_raise_system ();
-			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_SETGID, 0);
-		}
-
-		if (pw->pw_uid && setuid (pw->pw_uid) < 0) {
-			nih_error_raise_system ();
-			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_SETUID, 0);
-		}
-	}
-
 	/* Set the standard file descriptors to an output of our chosing;
 	 * any other open descriptor must be intended for the child, or have
 	 * the FD_CLOEXEC flag so it's automatically closed when we exec()
@@ -773,7 +723,8 @@ job_process_spawn (Job          *job,
 
 	/* Adjust the process priority ("nice level").
 	 */
-	if (setpriority (PRIO_PROCESS, 0, class->nice) < 0) {
+	if (class->nice != JOB_NICE_INVALID &&
+	    setpriority (PRIO_PROCESS, 0, class->nice) < 0) {
 		nih_error_raise_system ();
 		job_process_error_abort (fds[1],
 					 JOB_PROCESS_ERROR_PRIORITY, 0);
@@ -833,9 +784,11 @@ job_process_spawn (Job          *job,
 	 * configured in the job, or to the root directory of the filesystem
 	 * (or at least relative to the chroot).
 	 */
-	if (chdir (class->chdir ? class->chdir : user_job ? user_dir : "/") < 0) {
-		nih_error_raise_system ();
-		job_process_error_abort (fds[1], JOB_PROCESS_ERROR_CHDIR, 0);
+	if (class->chdir || user_mode == FALSE) {
+		if (chdir (class->chdir ? class->chdir : "/") < 0) {
+			nih_error_raise_system ();
+			job_process_error_abort (fds[1], JOB_PROCESS_ERROR_CHDIR, 0);
+		}
 	}
 
 	/* Change the user and group of the process to the one
@@ -844,7 +797,6 @@ job_process_spawn (Job          *job,
 	 * session jobs and jobs with a chroot stanza.
 	 */
 	if (class->setuid) {
-		struct passwd *pwd;
 		/* Without resetting errno, it's impossible to
 		 * distinguish between a non-existent user and and
 		 * error during lookup */
@@ -867,7 +819,6 @@ job_process_spawn (Job          *job,
 	}
 
 	if (class->setgid) {
-		struct group *grp;
 		errno = 0;
 		grp = getgrnam (class->setgid);
 		if (! grp) {
@@ -891,6 +842,35 @@ job_process_spawn (Job          *job,
 		job_process_error_abort (fds[1], JOB_PROCESS_ERROR_CHOWN, 0);
 	}
 
+	/* Make sure we always have the needed pwd and grp structs.
+	 * Then pass those to initgroups() to setup the user's group list.
+	 * Only do that if we're root as initgroups() won't work when non-root. */
+	if (geteuid () == 0) {
+		if (! pwd) {
+			pwd = getpwuid (geteuid ());
+			if (! pwd) {
+				nih_error_raise_system ();
+				job_process_error_abort (fds[1], JOB_PROCESS_ERROR_GETPWUID, 0);
+			}
+		}
+
+		if (! grp) {
+			grp = getgrgid (getegid ());
+			if (! grp) {
+				nih_error_raise_system ();
+				job_process_error_abort (fds[1], JOB_PROCESS_ERROR_GETGRGID, 0);
+			}
+		}
+
+		if (pwd && grp) {
+			if (initgroups (pwd->pw_name, grp->gr_gid) < 0) {
+				nih_error_raise_system ();
+				job_process_error_abort (fds[1], JOB_PROCESS_ERROR_INITGROUPS, 0);
+			}
+		}
+	}
+
+	/* Start dropping privileges */
 	if (job_setgid != (gid_t) -1 && setgid (job_setgid) < 0) {
 		nih_error_raise_system ();
 		job_process_error_abort (fds[1], JOB_PROCESS_ERROR_SETGID, 0);
@@ -1142,6 +1122,11 @@ job_process_error_read (int fd)
 				  err, _("unable to getpwuid: %s"),
 				  strerror (err->errnum)));
 		break;
+	case JOB_PROCESS_ERROR_GETGRGID:
+		err->error.message = NIH_MUST (nih_sprintf (
+				  err, _("unable to getgrgid: %s"),
+				  strerror (err->errnum)));
+		break;
 	case JOB_PROCESS_ERROR_BAD_SETUID:
 		err->error.message = NIH_MUST (nih_sprintf (
 				  err, _("unable to find setuid user")));
@@ -1200,6 +1185,11 @@ job_process_error_read (int fd)
 				  err, _("unable to allocate memory: %s"),
 				  strerror (err->errnum)));
 		break;
+	case JOB_PROCESS_ERROR_INITGROUPS:
+		err->error.message = NIH_MUST (nih_sprintf (
+				  err, _("unable to initgroups: %s"),
+				  strerror (err->errnum)));
+		break;
 	default:
 		nih_assert_not_reached ();
 	}
@@ -1249,6 +1239,52 @@ job_process_kill (Job         *job,
 	job_process_set_kill_timer (job, process, job->class->kill_timeout);
 }
 
+/**
+ * job_process_jobs_running:
+ *
+ * Determine if any jobs are running.
+ *
+ * Returns: TRUE if jobs are still running, else FALSE.
+ **/
+int
+job_process_jobs_running (void)
+{
+	job_class_init ();
+
+	NIH_HASH_FOREACH (job_classes, iter) {
+		JobClass *class = (JobClass *)iter;
+
+		NIH_HASH_FOREACH (class->instances, job_iter)
+			return TRUE;
+	}
+
+	return FALSE;
+}
+
+
+/**
+ * job_process_stop_all:
+ *
+ * Stop all running jobs.
+ **/
+void
+job_process_stop_all (void)
+{
+	job_class_init ();
+
+	NIH_HASH_FOREACH (job_classes, iter) {
+		JobClass *class = (JobClass *)iter;
+
+		/* Note that instances get killed in a random order */
+		NIH_HASH_FOREACH (class->instances, job_iter) {
+			Job *job = (Job *)job_iter;
+
+			/* Request job instance stops */
+			job_change_goal (job, JOB_STOP);
+		}
+	}
+}
+
 /**
  * job_process_set_kill_timer:
  * @job: job to set kill timer for,
@@ -1578,7 +1614,7 @@ job_process_terminated (Job         *job,
 			 * that's a simple matter of doing nothing.  Check
 			 * the job isn't running away first though.
 			 */
-			if (failed && job->class->respawn) {
+			if (failed && job->class->respawn && ! disable_respawn) {
 				if (job_process_catch_runaway (job)) {
 					nih_warn (_("%s respawning too fast, stopped"),
 						  job_name (job));
@@ -2140,7 +2176,7 @@ job_process_log_path (Job *job, int user_job)
 	nih_assert (class->name);
 
 	/* Override, primarily for tests */
-	if (getenv (LOGDIR_ENV)) {
+	if (getenv (LOGDIR_ENV) && ! user_mode) {
 		dir = nih_strdup (NULL, getenv (LOGDIR_ENV));
 		nih_debug ("Using alternative directory '%s' for logs", dir);
 	} else {
