@@ -30,6 +30,13 @@
 #include <sys/resource.h>
 #include <sys/mount.h>
 
+#ifdef HAVE_SYS_PRCTL_H
+#include <sys/prctl.h>
+#ifndef PR_SET_CHILD_SUBREAPER
+#define PR_SET_CHILD_SUBREAPER 35
+#endif
+#endif
+
 #include <errno.h>
 #include <stdio.h>
 #include <limits.h>
@@ -61,6 +68,7 @@
 #include "conf.h"
 #include "control.h"
 #include "state.h"
+#include "xdg.h"
 
 
 /* Prototypes for static functions */
@@ -112,6 +120,8 @@ static char *initial_event = NULL;
  **/
 static int disable_startup_event = FALSE;
 
+extern int          no_inherit_env;
+extern int          user_mode;
 extern int          disable_sessions;
 extern int          disable_job_logging;
 extern int          use_session_bus;
@@ -131,13 +141,16 @@ static NihOption options[] = {
 	{ 0, "default-console", N_("default value for console stanza"),
 		NULL, "VALUE", NULL, console_type_setter },
 
+	{ 0, "no-inherit-env", N_("jobs will not inherit environment of init"),
+		NULL, NULL, &no_inherit_env ,NULL },
+
 	{ 0, "logdir", N_("specify alternative directory to store job output logs in"),
 		NULL, "DIR", &log_dir, NULL },
 
 	{ 0, "no-log", N_("disable job logging"),
 		NULL, NULL, &disable_job_logging, NULL },
 
-	{ 0, "no-sessions", N_("disable user and chroot sessions"),
+	{ 0, "no-sessions", N_("disable chroot sessions"),
 		NULL, NULL, &disable_sessions, NULL },
 
 	{ 0, "no-startup-event", N_("do not emit any startup event (for testing)"),
@@ -157,6 +170,9 @@ static NihOption options[] = {
 	{ 0, "startup-event", N_("specify an alternative initial event (for testing)"),
 		NULL, "NAME", &initial_event, NULL },
 
+	{ 0, "user", N_("start in user mode (as used for user sessions)"),
+		NULL, NULL, &user_mode, NULL },
+
 	/* Ignore invalid options */
 	{ '-', "--", NULL, NULL, NULL, NULL, NULL },
 
@@ -169,6 +185,7 @@ main (int   argc,
       char *argv[])
 {
 	char **args = NULL;
+	char **dirs = NULL;
 	int    ret;
 
 	args_copy = NIH_MUST (nih_str_array_copy (NULL, NULL, argv));
@@ -193,8 +210,11 @@ main (int   argc,
 
 	control_handle_bus_type ();
 
+	if (! user_mode)
+		no_inherit_env = TRUE;
+
 #ifndef DEBUG
-	if (use_session_bus == FALSE) {
+	if (use_session_bus == FALSE && user_mode == FALSE) {
 
 		int needs_devtmpfs = 0;
 
@@ -370,7 +390,7 @@ main (int   argc,
 		nih_signal_reset ();
 
 #ifndef DEBUG
-	if (use_session_bus == FALSE) {
+	if (use_session_bus == FALSE && user_mode == FALSE) {
 		/* Catch fatal errors immediately rather than waiting for a new
 		 * iteration through the main loop.
 		 */
@@ -387,7 +407,7 @@ main (int   argc,
 	nih_signal_set_handler (SIGALRM, nih_signal_handler);
 
 #ifndef DEBUG
-	if (use_session_bus == FALSE) {
+	if (use_session_bus == FALSE && user_mode == FALSE) {
 		/* Ask the kernel to send us SIGINT when control-alt-delete is
 		 * pressed; generate an event with the same name.
 		 */
@@ -408,19 +428,20 @@ main (int   argc,
 		nih_signal_set_handler (SIGPWR, nih_signal_handler);
 		NIH_MUST (nih_signal_add_handler (NULL, SIGPWR, pwr_handler, NULL));
 
-		/* SIGHUP instructs us to re-load our configuration */
-		nih_signal_set_handler (SIGHUP, nih_signal_handler);
-		NIH_MUST (nih_signal_add_handler (NULL, SIGHUP, hup_handler, NULL));
-
-		/* SIGUSR1 instructs us to reconnect to D-Bus */
-		nih_signal_set_handler (SIGUSR1, nih_signal_handler);
-		NIH_MUST (nih_signal_add_handler (NULL, SIGUSR1, usr1_handler, NULL));
-
 	}
 
-	/* SIGTERM instructs us to re-exec ourselves; this should be the
-	 * last in the list to ensure that all other signals are handled
-	 * before a SIGTERM.
+	/* SIGHUP instructs us to re-load our configuration */
+	nih_signal_set_handler (SIGHUP, nih_signal_handler);
+	NIH_MUST (nih_signal_add_handler (NULL, SIGHUP, hup_handler, NULL));
+
+	/* SIGUSR1 instructs us to reconnect to D-Bus */
+	nih_signal_set_handler (SIGUSR1, nih_signal_handler);
+	NIH_MUST (nih_signal_add_handler (NULL, SIGUSR1, usr1_handler, NULL));
+
+	/* SIGTERM instructs us to re-exec ourselves when running as PID
+	 * 1, or to exit when running as a Session Init; this signal should
+	 * be the last in the list to ensure that all other signals are
+	 * handled before a SIGTERM.
 	 */
 	nih_signal_set_handler (SIGTERM, nih_signal_handler);
 	NIH_MUST (nih_signal_add_handler (NULL, SIGTERM, term_handler, NULL));
@@ -474,7 +495,6 @@ main (int   argc,
 			nih_warn ("%s",
 				_("Stateful re-exec supported but stateless re-exec requested"));
 		} else if (state_read (state_fd) < 0) {
-			nih_local char *arg = NULL;
 
 			/* Stateful re-exec has failed so try once more by
 			 * degrading to stateless re-exec, which even in
@@ -512,8 +532,20 @@ main (int   argc,
 	}
 
 	/* Read configuration */
-	NIH_MUST (conf_source_new (NULL, CONFFILE, CONF_FILE));
-	NIH_MUST (conf_source_new (NULL, conf_dir, CONF_JOB_DIR));
+	if (! user_mode)
+		NIH_MUST (conf_source_new (NULL, CONFFILE, CONF_FILE));
+
+	if (conf_dir)
+		NIH_MUST (conf_source_new (NULL, conf_dir, CONF_JOB_DIR));
+
+	if (user_mode) {
+		dirs = NIH_MUST (get_user_upstart_dirs ());
+		for (char **d = dirs; d && *d; d++)
+			NIH_MUST (conf_source_new (NULL, *d, CONF_JOB_DIR));
+		nih_free (dirs);
+	}
+
+	job_class_environment_init ();
 
 	conf_reload ();
 
@@ -550,7 +582,7 @@ main (int   argc,
 	}
 
 #ifndef DEBUG
-	if (use_session_bus == FALSE) {
+	if (use_session_bus == FALSE && user_mode == FALSE) {
 		/* Now that the startup is complete, send all further logging output
 		 * to kmsg instead of to the console.
 		 */
@@ -598,10 +630,28 @@ main (int   argc,
 		 * disabled by the term_handler */
 		sigemptyset (&mask);
 		sigprocmask (SIG_SETMASK, &mask, NULL);
+
+		/* Emit the Restarted signal so that any listing Instance Init
+		 * knows that it needs to restart too.
+		 */
+		control_notify_restarted();
 	}
 
 	if (disable_sessions)
-		nih_debug ("Sessions disabled");
+		nih_debug ("Chroot Sessions disabled");
+
+	/* Set us as the child subreaper.
+	 * This ensures that even when init doesn't run as PID 1, it'll always be
+	 * the ultimate parent of everything it spawns. */
+
+#ifdef HAVE_SYS_PRCTL_H
+	if (getpid () > 1 && prctl (PR_SET_CHILD_SUBREAPER, 1) < 0) {
+		nih_warn ("%s: %s", _("Unable to register as subreaper"),
+				  strerror (errno));
+
+		NIH_MUST (event_new (NULL, "child-subreaper-failed", NULL));
+	}
+#endif
 
 	/* Run through the loop at least once to deal with signals that were
 	 * delivered to the previous process while the mask was set or to
@@ -757,7 +807,8 @@ crash_handler (int signum)
  * @signal: signal caught.
  *
  * This is called when we receive the TERM signal, which instructs us
- * to reexec ourselves.
+ * to reexec ourselves when running as PID 1, or to perform a controlled
+ * exit when running as a Session Init.
  **/
 static void
 term_handler (void      *data,
@@ -766,8 +817,12 @@ term_handler (void      *data,
 	nih_assert (args_copy[0] != NULL);
 	nih_assert (signal != NULL);
 
-	nih_warn (_("Re-executing %s"), args_copy[0]);
+	if (user_mode) {
+		quiesce (QUIESCE_REQUESTER_SYSTEM);
+		return;
+	}
 
+	nih_warn (_("Re-executing %s"), args_copy[0]);
 	stateful_reexec ();
 }
 
@@ -778,7 +833,7 @@ term_handler (void      *data,
  * @data: unused,
  * @signal: signal that called this handler.
  *
- * Handle having recieved the SIGINT signal, sent to us when somebody
+ * Handle having received the SIGINT signal, sent to us when somebody
  * presses Ctrl-Alt-Delete on the console.  We just generate a
  * ctrlaltdel event.
  **/
@@ -794,7 +849,7 @@ cad_handler (void      *data,
  * @data: unused,
  * @signal: signal that called this handler.
  *
- * Handle having recieved the SIGWINCH signal, sent to us when somebody
+ * Handle having received the SIGWINCH signal, sent to us when somebody
  * presses Alt-UpArrow on the console.  We just generate a
  * kbdrequest event.
  **/
@@ -810,7 +865,7 @@ kbd_handler (void      *data,
  * @data: unused,
  * @signal: signal that called this handler.
  *
- * Handle having recieved the SIGPWR signal, sent to us when powstatd
+ * Handle having received the SIGPWR signal, sent to us when powstatd
  * changes the /etc/powerstatus file.  We just generate a
  * power-status-changed event and jobs read the file.
  **/
@@ -826,7 +881,7 @@ pwr_handler (void      *data,
  * @data: unused,
  * @signal: signal that called this handler.
  *
- * Handle having recieved the SIGHUP signal, which we use to instruct us to
+ * Handle having received the SIGHUP signal, which we use to instruct us to
  * reload our configuration.
  **/
 static void
@@ -842,7 +897,7 @@ hup_handler (void      *data,
  * @data: unused,
  * @signal: signal that called this handler.
  *
- * Handle having recieved the SIGUSR signal, which we use to instruct us to
+ * Handle having received the SIGUSR signal, which we use to instruct us to
  * reconnect to D-Bus.
  **/
 static void
@@ -878,6 +933,9 @@ handle_confdir (void)
 	if (conf_dir)
 		goto out;
 
+	if (user_mode)
+		return;
+
 	conf_dir = CONFDIR;
 
 	dir = getenv (CONFDIR_ENV);
@@ -905,6 +963,11 @@ handle_logdir (void)
 	if (log_dir)
 		goto out;
 
+	if (user_mode) {
+		log_dir = get_user_log_dir ();
+		return;
+	}
+
 	log_dir = JOB_LOGDIR;
 
 	dir = getenv (LOGDIR_ENV);
@@ -938,4 +1001,3 @@ console_type_setter (NihOption *option, const char *arg)
 
 	 return 0;
 }
-
