@@ -71,20 +71,20 @@ typedef struct process {
 NIH_BEGIN_EXTERN
 
 Process *   process_new       (const void *parent)
-	__attribute__ ((warn_unused_result, malloc));
+	__attribute__ ((warn_unused_result));
 
 const char *process_name      (ProcessType process)
 	__attribute__ ((const));
 
 json_object *process_serialise (const Process *process)
-	__attribute__ ((malloc, warn_unused_result));
+	__attribute__ ((warn_unused_result));
 
 Process *process_deserialise (json_object *json, const void *parent)
-	__attribute__ ((malloc, warn_unused_result));
+	__attribute__ ((warn_unused_result));
 
 json_object *
 process_serialise_all (const Process * const * const processes)
-	__attribute__ ((malloc, warn_unused_result));
+	__attribute__ ((warn_unused_result));
 
 ProcessType process_from_name (const char *process)
 	__attribute__ ((warn_unused_result));
